### Miscellaneous notes

Unicode is a huge standard; implementing the whole of Unicode is a big task, way beyond the scope of this initial version of Fenius. Basic UTF-8 support such as counting the number of codepoints in a string, fetching the n-th codepoint, iterating over codepoints, etc., must be supported, but anything involving knowledge of the properties of individual characters (such as lowercasing string with non-ASCII characters, or iterating over graphemes as opposed to codepoints) may be left for a future version of the language (although if the host implementation provides facilities for handling those cases, these can be used). The goal for this report is to define a reasonable API that can be extended later with full Unicode support.

Handling strings that are not valid UTF-8 is important because data coming from the operating system, such as filenames and program arguments, should be representable as strings, and on Unix such strings can contain any bytes (except the ASCII NUL byte, which nonetheless must also be supported in strings). (The situation in Windows, where such strings are UTF-16 but allow unpaired surrogate codepoints to appear, is more complex and not addressed by this report.)

I think it's fair to look up to [Elixir](https://hexdocs.pm/elixir/String.html) as the gold standard when it comes to correct handling of Unicode, especially when invalid UTF-8 must also be accepted. [Julia](https://docs.julialang.org/en/v1/manual/strings/) may also have some ideas to offer here, and [Swift](https://developer.apple.com/documentation/swift/string) may also have useful ideas in the direction of making strings representation-independent. On the weirder side of things, Raku (formerly Perl 6) actually [encodes strings as sequences of graphemes](https://github.com/MoarVM/MoarVM/blob/master/docs/strings.asciidoc) by generating custom characters for grapheme clusters on the fly, but that's far away from the way I want to handle things in Fenius.

Unlike languages like C, Scheme and Common Lisp, Fenius does not have a distinct character or codepoint type. Instead, single-character strings are used in most places. This has the advantage of being able to represent both valid Unicode characters and non-UTF-8 bytes in a uniform way (as strings). Characters as a datatype are overrated anyway, especially if they correspond to Unicode codepoints: most operations typically performed on Unicode codepoints should be performed on grapheme clusters instead. For instance, the reverse of the string "🇸🇪🇮🇸" (composed of Unicode codepoints for the regional indicator symbol letters S, E, I, S) should be "🇮🇸🇸🇪", not "🇸🇮🇪🇸" (which is what you get by putting the Unicode codepoints in reverse order). And if the character datatype is going to be more complex than a Unicode codepoint, one might as well represent them as strings anyway.
